@@ -850,6 +850,16 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 	return cp->sbuf;
 }
 
+/*
+ * Set db node id to connection.
+ */
+void pool_set_db_node_id(POOL_CONNECTION *con, int db_node_id)
+{
+	if (!con)
+		return;
+	con->db_node_id = db_node_id;
+}
+
 /*
  * returns the byte length of str, including \0, no more than upper.
  * if encountered \0, flag is set to non 0.
@@ -982,6 +992,7 @@ int pool_unread(POOL_CONNECTION *cp, void *data, int len)
         MemoryContextSwitchTo(oldContext);
 		
         cp->hp = p;
+		cp->bufsz = realloc_size;
 	}
 	if (cp->len != 0)
 		memmove(p + len, cp->hp + cp->po, cp->len);
